@@ -14,6 +14,8 @@ ZoneType = Literal["intersect", "contain"]
 ZoneFilter = Literal["include", "exclude"]
 ObjectClass = Literal["person", "vehicle", "animal", "motion", "other"]
 CameraRole = Literal["high-resolution", "mid-resolution", "low-resolution", "snapshot"]
+StreamingRole = Literal["high-resolution", "mid-resolution", "low-resolution"]
+VideoStreamingMode = Literal["auto", "webrtc", "mse", "webrtc/tcp"]
 DecoderFormat = Literal["yuv420p", "rgb24", "nv12"]
 ImageInputFormat = Literal["yuv420p", "nv12", "rgb", "rgba", "gray"]
 ImageOutputFormat = Literal["rgb", "rgba", "gray"]
@@ -27,38 +29,38 @@ CameraFrameWorkerResolution = Literal[
     "320x240",
     "320x180",
 ]
-AudioCodec = Literal["PCMU", "PCMA", "MPEG4-GENERIC", "opus", "G722", "MPA", "PCM", "FLAC"]
+AudioCodec = Literal[
+    "PCMU", "PCMA", "MPEG4-GENERIC", "opus", "G722", "MPA", "PCM", "FLAC", "ELD", "PCML", "L16"
+]
 AudioFFmpegCodec = Literal[
     "pcm_mulaw", "pcm_alaw", "aac", "libopus", "g722", "mp3", "pcm_s16be", "pcm_s16le", "flac"
 ]
 VideoCodec = Literal["H264", "H265", "VP8", "VP9", "AV1", "JPEG", "RAW"]
 VideoFFmpegCodec = Literal["h264", "hevc", "vp8", "vp9", "av1", "mjpeg", "rawvideo"]
 PythonVersion = Literal["3.9", "3.10", "3.11", "3.12"]
-InternalSourceType = Literal["aac", "opus", "pcma"]
 StateNames = Literal["light", "motion", "audio", "doorbell", "siren", "battery", "object"]
+# See Camera
 CameraPublicProperties = Literal[
     "_id",
-    "audioDetectionPluginId",
-    "cameraControllerPluginId",
-    "detectionSettings",
-    "detectionZones",
+    "nativeId",
+    "pluginId",
+    "name",
     "disabled",
-    "frameWorkerSettings",
-    "hasBattery",
-    "hasBinarySensor",
+    "isCloud",
     "hasLight",
     "hasSiren",
+    "hasBinarySensor",
+    "hasBattery",
     "info",
-    "isCloud",
-    "motionDetectionPluginId",
-    "name",
-    "nativeId",
-    "objectDetectionPluginId",
-    "pluginId",
-    "ptzPluginId",
-    "snapshotTTL",
-    "sources",
     "type",
+    "snapshotTTL",
+    "detectionSettings",
+    "frameWorkerSettings",
+    "interface",
+    "recording",
+    "extensions",
+    "sources",
+    "detectionZones",
 ]
 DeviceManagerEventType = Literal[
     "cameraSelected",
@@ -67,7 +69,7 @@ DeviceManagerEventType = Literal[
 JsonSchemaType = Literal["string", "number", "boolean", "object", "array", "button"]
 LoggerLevel = Literal["error", "warn", "info", "debug", "trace", "attention", "success"]
 APIEventType = Literal["finishLaunching", "shutdown"]
-InternalRole = Literal["AAC", "OPUS", "PCMA"]
+PTZMovementType = Literal["absolute", "relative", "continuous", "home", "preset", "stop"]
 
 # Basic types
 Callback = Union[
@@ -106,26 +108,6 @@ class Detection(TypedDict):
     origHeight: int
 
 
-class ZoneCoord(TypedDict):
-    _id: str
-    points: Point
-
-
-class ZoneRegion(TypedDict):
-    _id: str
-    coords: list[ZoneCoord]
-    type: ZoneType
-    filter: ZoneFilter
-    classes: list[ObjectClass]
-    isPrivacyMask: bool
-    color: str
-
-
-class CameraZone(TypedDict):
-    name: str
-    regions: list[ZoneRegion]
-
-
 class DetectionZone(TypedDict):
     name: str
     points: list[Point]
@@ -162,7 +144,6 @@ class CameraInput(TypedDict):
     useForSnapshot: bool
     hotMode: bool
     urls: "StreamUrls"
-    internal: Optional[InternalRole]
 
 
 class RTPInfo(TypedDict):
@@ -170,6 +151,8 @@ class RTPInfo(TypedDict):
     codec: str
     rate: Optional[int]
     encoding: Optional[int]
+    profile: Optional[str]
+    level: Optional[int]
 
 
 class FMTPInfo(TypedDict):
@@ -205,9 +188,9 @@ class VideoStreamInfo(TypedDict):
 
 
 class ProbeConfig(TypedDict, total=False):
-    video: VideoCodec
-    audio: AudioCodec
-    microphone: AudioCodec
+    video: Union[Literal["all"], list[VideoCodec]]
+    audio: Union[Literal["all"], list[AudioCodec]]
+    microphone: Union[Literal["all"], list[AudioCodec]]
 
 
 class ProbeStream(TypedDict):
@@ -219,41 +202,7 @@ class ProbeStream(TypedDict):
 class StreamUrls(TypedDict):
     ws: "Go2RtcWSSource"
     rtsp: "Go2RtcRTSPSource"
-    www: "Go2RtcEndpoint"
-
-
-class Go2RtcRTSPSource(TypedDict):
-    default: str
-    defaultMicrophone: str
-    muted: str
-    h264: str
-    h265: str
-    aac: str
-    opus: str
-    pcma: str
-    mp4: str
-    onvif: str
-
-
-class Go2RtcEndpoint(TypedDict):
-    webrtc: str
-    mse: str
-    lmp4: str
-    mmp4: str
-    mp4: str
-    mp4Snapshot: str
-    jpegSnapshot: str
-    lHlsTs: str
-    lHlsFmp4: str
-    mHlsFmp4: str
-    mjpeg: str
-    mjpegHtml: str
-
-
-class Go2RtcSource(TypedDict):
-    name: str
-    src: str
-    ws: str
+    snapshot: "Go2RtcSnapshotSource"
 
 
 class Go2RtcWSSource(TypedDict):
@@ -261,6 +210,22 @@ class Go2RtcWSSource(TypedDict):
     mse: str
 
 
+class Go2RtcRTSPSource(TypedDict):
+    base: str
+    default: str
+    muted: str
+    aac: str
+    opus: str
+    pcma: str
+    onvif: str
+
+
+class Go2RtcSnapshotSource(TypedDict):
+    mp4: str
+    jpeg: str
+    mjpeg: str
+
+
 T = TypeVar(
     "T",
     bound=Union[
@@ -483,7 +448,6 @@ class CameraInputSettings(TypedDict):
     useForSnapshot: bool
     hotMode: bool
     urls: list[str]
-    internal: Optional[InternalRole]
 
 
 class CameraConfigInputSettings(TypedDict):
@@ -514,11 +478,49 @@ class CameraDelegate(Protocol):
     async def snapshot(self, source_id: str, force_new: Optional[bool] = None) -> Optional[bytes]: ...
 
 
+class PTZSpeed(TypedDict):
+    pan: NotRequired[float]
+    tilt: NotRequired[float]
+    zoom: NotRequired[float]
+
+
+class PTZBaseCommand(TypedDict):
+    speed: NotRequired[PTZSpeed]
+
+
+class PTZStopCommand(PTZBaseCommand):
+    type: Literal["stop"]
+
+
+class PTZHomeCommand(PTZBaseCommand):
+    type: Literal["home"]
+
+
+class PTZPresetCommand(PTZBaseCommand):
+    type: Literal["preset"]
+    preset: str
+
+
+class PTZMoveCommand(PTZBaseCommand):
+    type: Literal["absolute", "relative"]
+    pan: NotRequired[float]
+    tilt: NotRequired[float]
+    zoom: NotRequired[float]
+
+
+class PTZContinuousMoveCommand(PTZBaseCommand):
+    type: Literal["continuous"]
+    pan: NotRequired[float]
+    tilt: NotRequired[float]
+    zoom: NotRequired[float]
+    timeout: NotRequired[float]
+
+
+PTZCommand = Union[PTZHomeCommand, PTZPresetCommand, PTZMoveCommand, PTZContinuousMoveCommand]
+
+
 class CameraPTZDelegate(Protocol):
-    async def moveAbsolute(self, pan: float, tilt: float, zoom: float) -> None: ...
-    async def moveRelative(self, pan: float, tilt: float, zoom: float) -> None: ...
-    async def moveContinuous(self, pan: float, tilt: float, zoom: float) -> None: ...
-    async def stop(self) -> None: ...
+    async def ptzCommand(self, command: PTZCommand) -> None: ...
 
 
 class CameraDelegates(TypedDict):
@@ -526,7 +528,15 @@ class CameraDelegates(TypedDict):
     ptzDelegate: CameraPTZDelegate
 
 
-class CameraBaseSource(Protocol):
+class RTSPUrlOptions(TypedDict, total=False):
+    video: bool
+    audio: Union[bool, str, list[str]]
+    audioSingleTrack: bool
+    backchannel: bool
+    timeout: int
+
+
+class CameraSource(Protocol):
     # from CameraInput
     id: str  # _id
     name: str
@@ -534,7 +544,6 @@ class CameraBaseSource(Protocol):
     useForSnapshot: bool
     hotMode: bool
     urls: StreamUrls
-    internal: Optional[InternalRole]
     # end CameraInput
 
     async def snapshot(self, force_new: Optional[bool] = None) -> Optional[bytes]: ...
@@ -543,15 +552,14 @@ class CameraBaseSource(Protocol):
         self, probe_config: Optional[ProbeConfig] = None, refresh: Optional[bool] = None
     ) -> Optional[ProbeStream]: ...
 
+
+class CameraDeviceSource(CameraSource, Protocol):
+    def generate_rtsp_url(self, options: RTSPUrlOptions) -> str: ...
     def create_webrtc_session(self) -> None: ...
     def create_rtsp_session(self) -> None: ...
     def create_fmp4_session(self) -> None: ...
 
 
-class CameraDeviceSource(CameraBaseSource, Protocol):
-    def get_internal_source(self, type: InternalSourceType) -> "CameraBaseSource": ...
-
-
 class WebRTCConnectionOptions(TypedDict, total=False):
     iceServers: list["IceServer"]
 
@@ -559,19 +567,18 @@ class WebRTCConnectionOptions(TypedDict, total=False):
 class RTSPConnectionOptions(TypedDict, total=False):
     target: Literal[
         "default",
-        "defaultMicrophone",
-        "h264",
-        "h265",
         "aac",
         "opus",
         "pcma",
-        "h264aac",
-        "h265aac",
-        "h264opus",
-        "h265opus",
-        "h264pcma",
-        "h265pcma",
-        "mp4",
+    ]
+
+
+class FMP4ConnectionOptions(TypedDict, total=False):
+    target: Literal[
+        "default",
+        "aac",
+        "opus",
+        "pcma",
     ]
 
 
@@ -593,7 +600,31 @@ class ReturnAudioFFmpegOptions(TypedDict):
     logPrefix: Optional[str]
 
 
-class BaseCamera(TypedDict):
+class CameraUiSettings(TypedDict):
+    streamingMode: VideoStreamingMode
+    streamingSource: Union[StreamingRole, Literal["auto"]]
+
+
+class CameraRecordingSettings(TypedDict):
+    enabled: bool
+
+
+class Extension(TypedDict):
+    id: str
+    name: str
+
+
+class CameraExtensions(TypedDict):
+    hub: NotRequired[list[Extension]]
+    cameraController: NotRequired[Extension]
+    motionDetection: NotRequired[Extension]
+    audioDetection: NotRequired[Extension]
+    objectDetection: NotRequired[Extension]
+    ptz: NotRequired[Extension]
+    plugins: list[Extension]
+
+
+class Camera(TypedDict):
     _id: str
     nativeId: Optional[str]
     pluginId: str
@@ -607,18 +638,13 @@ class BaseCamera(TypedDict):
     info: CameraInformation
     type: CameraType
     snapshotTTL: int
+    detectionZones: list[DetectionZone]
     detectionSettings: CameraDetectionSettings
     frameWorkerSettings: CameraFrameWorkerSettings
-
-
-class Camera(BaseCamera):
-    cameraControllerPluginId: Optional[str]
-    audioDetectionPluginId: Optional[str]
-    motionDetectionPluginId: Optional[str]
-    objectDetectionPluginId: Optional[str]
-    ptzPluginId: Optional[str]
+    interface: CameraUiSettings
+    recording: CameraRecordingSettings
+    extensions: CameraExtensions
     sources: list[CameraInput]
-    detectionZones: list[DetectionZone]
 
 
 StateValue = Union[
@@ -709,13 +735,11 @@ class CameraDevice(Protocol):
     @property
     def low_resolution_source(self) -> Optional[CameraDeviceSource]: ...
     @property
-    def snapshot_source(self) -> Optional[CameraBaseSource]: ...
+    def snapshot_source(self) -> Optional[CameraSource]: ...
     @property
     def ptz(self) -> CameraPTZDelegate: ...
     @property
     def sources(self) -> list[CameraDeviceSource]: ...
-    @property
-    def internal_sources(self) -> list[CameraBaseSource]: ...
 
     on_connected: HybridObservable[bool]
     on_frameworker_connected: HybridObservable[bool]
@@ -953,7 +977,7 @@ class JsonSchemaString(JsonBaseSchema[str]):
     maxLength: NotRequired[int]
 
 
-class JsonSchemaNumber(JsonBaseSchema[float]):
+class JsonSchemaNumber(JsonBaseSchema[Union[int, float]]):
     type: Literal["number"]
     minimum: NotRequired[float]
     maximum: NotRequired[float]
@@ -964,19 +988,19 @@ class JsonSchemaBoolean(JsonBaseSchema[bool]):
     type: Literal["boolean"]
 
 
-class JsonSchemaEnum(JsonBaseSchema[str]):
+class JsonSchemaEnum(JsonBaseSchema[Union[str, list[str]]]):
     type: Literal["string"]
     enum: list[str]
     multiple: NotRequired[bool]
 
 
-class JsonSchemaObject(JsonBaseSchema[Any]):
+class JsonSchemaObject(JsonBaseSchema[dict[Any, Any]]):
     type: Literal["object"]
     opened: NotRequired[bool]
     properties: NotRequired["JsonSchemaForm"]
 
 
-class JsonSchemaArray(JsonBaseSchema[Any]):
+class JsonSchemaArray(JsonBaseSchema[list[Any]]):
     type: Literal["array"]
     opened: NotRequired[bool]
     items: NotRequired["JsonSchema"]
